<?php
// $Horde: nag/lib/Nag.php,v 1.16 2001/06/04 21:49:12 chuck Exp $

/** @const NAG_NAME The sitename. */
define('NAG_NAME', 'Task List Manager');

require_once NAG_BASE . '/lib/version.php';

// Actions
/** @const NAG_ADD_TASK Add a new task.           */ define('NAG_ADD_TASK', 100);
/** @const NAG_DELETE_TASKS Delete tasks.         */ define('NAG_DELETE_TASKS', 101);
/** @const NAG_MODIFY_TASK Modify a task.         */ define('NAG_MODIFY_TASK', 102);
/** @const NAG_SAVE_TASK Save a task to storage.  */ define('NAG_SAVE_TASK', 103);
/** @const NAG_SEARCH_TASKS Search the task list. */ define('NAG_SEARCH_TASKS', 104);

// Sort types
/** @const SORT_NAME Sort by task name.         */ define('SORT_NAME', 0);
/** @const SORT_ADDED Sort by creation date.    */ define('SORT_ADDED', 1);
/** @const SORT_DUE Sort by due date.           */ define('SORT_DUE', 2);
/** @const SORT_ASCEND Sort in ascending order.  */ define('SORT_ASCEND', 0);
/** @const SORT_DESCEND Sort in descending order.*/ define('SORT_DESCEND', 1);

/* Set the umask. */
if (isset($conf['umask'])) {
    umask($conf['umask']);
}

/**
 * Nag Base Class.
 *
 * @author Jon Parise <jon@horde.org>
 * @version $Revision: 1.16 $
 * @package nag
 */
class Nag {

    /**
     * Retrieves the current user's task list from storage.
     * This function will also sort the resulting list, if requested.
     *
     * @param $storage      The current storage object.
     * @param $criteria     (optional) A set of flags specifying the set of
     *                      tasks to return. (TASK_ADDED, TASK_MODIFIED,
     *                      TASK_DELETED)
     * @param $sortby       (optional) The field by which to sort.
     *                      (SORT_NAME, SORT_ADDED, SORT_DUE)
     * @param $sortdir      (optional) The direction by which to sort.
     *                      (SORT_ASC, SORT_DESC)
     *
     * @return array        Returns a list of the requested tasks.
     *
     * @see Nag_Storage::listTasks
     */
    function listTasks($storage, $criteria = 0, $sortby = SORT_ADDED,
                       $sortdir = SORT_ASCEND)
    {
        /* Retrieve the task list for storage. */
        $tasks = $storage->listTasks($criteria);

        /* Sort the task list. */
        $sort_functions = array(
            SORT_NAME => 'ByName',
            SORT_ADDED => 'ByAdded',
            SORT_DUE => 'ByDue'
        );

        /* Sort the array if we have a sort function defined for this field. */
        if (isset($sort_functions[$sortby])) {
            $prefix = ($sortdir == SORT_DESCEND) ? '_rsort' : '_sort';
            uasort($tasks, $prefix . $sort_functions[$sortby]);
        }

        return $tasks;
    }

    /**
     * Generates the HTML for a day selection widget.
     *
     * @param $name         The name of the widget.
     * @param $timestamp    (optional) The timestamp to select by default.
     *
     * @return string       The HTML <select> widget.
     */
    function buildDayWidget($name, $timestamp = null)
    {
        $selected = (isset($timestamp)) ? date('d', $timestamp) : null;

        $html = "<select name=\"$name\">";
        for ($day = 1; $day <= 31; $day++) {
            $html .= "<option value=\"$day\"";
            $html .= ($day == $selected) ? ' selected>' : '>';
            $html .= "$day</option>";
        }
        $html .= '</select>';

        return $html;
    }

    /**
     * Generates the HTML for a month selection widget.
     *
     * @param $name         The name of the widget.
     * @param $timestamp    (optional) The timestamp to select by default.
     *
     * @return string       The HTML <select> widget.
     */
    function buildMonthWidget($name, $timestamp = null)
    {
        $months = array(
            1 => _("January"),
            2 => _("February"),
            3 => _("March"),
            4 => _("April"),
            5 => _("May"),
            6 => _("June"),
            7 => _("July"),
            8 => _("August"),
            9 => _("September"),
            10 => _("October"),
            11 => _("November"),
            12 => _("December")
        );

        $selected = (isset($timestamp)) ? date('m', $timestamp) : null;

        $html = "<select name=\"$name\">";
        for ($month = 1; $month <= 12; $month++) {
            $html .= "<option value=\"$month\"";
            $html .= ($month == $selected) ? ' selected>' : '>';
            $html .= $months[$month] . '</option>';
        }
        $html .= '</select>';

        return $html;
    }

    /**
     * Generates the HTML for a year selection widget.
     *
     * @param $name         The name of the widget.
     * @param $years        The number of years into the future to include.
     * @param $timestamp    (optional) The timestamp to select by default.
     *
     * @return string       The HTML <select> widget.
     */
    function buildYearWidget($name, $years, $timestamp = null)
    {
        $selected = (isset($timestamp)) ? date('Y', $timestamp) : null;

        $html = "<select name=\"$name\">";
        for ($year = date('Y'); $year < (date('Y') + $years); $year++) {
            $html .= "<option value=\"$year\"";
            $html .= ($year == $selected) ? ' selected>' : '>';
            $html .= "$year</option>";
        }
        $html .= '</select>';

        return $html;
    }

    /**
     * Generates the HTML for an hour selection widget.
     *
     * @param $name         The name of the widget.
     * @param $timestamp    (optional) The timestamp to select by default.
     *
     * @return string       The HTML <select> widget.
     */
    function buildHourWidget($name, $timestamp = null)
    {
        $selected = (isset($timestamp)) ? date('H', $timestamp) : null;

        $html = "<select name=\"$name\">";
        for ($hour = 0; $hour <= 23; $hour++) {
            $html .= "<option value=\"$hour\"";
            $html .= ($hour == $selected) ? ' selected>' : '>';
            $html .= sprintf("%02d", $hour) . '</option>';
        }
        $html .= '</select>';

        return $html;
    }

    /**
     * Generates the HTML for a minute selection widget.
     *
     * @param $name         The name of the widget.
     * @param $increment    (optional) The increment between minutes.
     * @param $timestamp    (optional) The timestamp to select by default.
     *
     * @return string       The HTML <select> widget.
     */
    function buildMinuteWidget($name, $increment = 1, $timestamp = null)
    {
        $selected = (isset($timestamp)) ? date('i', $timestamp) : null;

        $html = "<select name=\"$name\">";
        for ($minute = 0; $minute <= 60; $minute += $increment) {
            $html .= "<option value=\"$minute\"";
            $html .= ($minute == $selected) ? ' selected>' : '>';
            $html .= sprintf("%02d", $minute) . '</option>';
        }
        $html .= '</select>';

        return $html;
    }
}

function _sortByName($a, $b)
{
    return strcmp($a['name'], $b['name']);
}

function _rsortByName($a, $b)
{
    return strcmp($b['name'], $a['name']);
}

function _sortByAdded($a, $b)
{
    if ($a['added'] == $b['added']) return 0;
    return ($a['added'] > $b['added']) ? -1 : 1;
}

function _rsortByAdded($a, $b)
{
    if ($a['added'] == $b['added']) return 0;
    return ($a['added'] < $b['added']) ? -1 : 1;
}

function _sortByDue($a, $b)
{
    if ($a['due'] == $b['due']) return 0;
    return ($a['due'] > $b['due']) ? -1 : 1;
}

function _rsortByDue($a, $b)
{
    if ($a['due'] == $b['due']) return 0;
    return ($a['due'] < $b['due']) ? -1 : 1;
}

?>
